from rich.jupyter import JupyterMixin
from rich.ansi import AnsiDecoder
from rich.console import Group


class PlotextMixin(JupyterMixin):
    def __init__(self, phase=0, title="", make_plot=None):
        self.decoder = AnsiDecoder()
        self.phase = phase
        self.title = title
        self.make_plot = make_plot

    def __rich_console__(self, console, options):
        self.width = options.max_width or console.width
        self.height = options.height or console.height
        canvas = self.make_plot(self.width, self.height)
        self.rich_canvas = Group(*self.decoder.decode(canvas))
        yield self.rich_canvas



import json

import argcomplete
import importlib.metadata

from mando import command, main, arg
import time
from datetime import datetime
import sys
from copy import deepcopy

from ketacli.sdk.base.client import *
from ketacli.sdk.base.search import search_spl
from ketacli.sdk.request.list import list_assets_request, list_admin_request
from ketacli.sdk.request.create import create_asset_request
from ketacli.sdk.request.get import get_asset_by_id_request
from ketacli.sdk.request.update import update_asset_request
from ketacli.sdk.request.delete import delete_asset_request
from ketacli.sdk.request.asset_map import get_resources, get_resource

from ketacli.sdk.base.config import *

from ketacli.sdk.output.output import list_output, describe_output, get_asset_output
from ketacli.sdk.output.output import search_result_output
from ketacli.sdk.output.format import format_table
from ketacli.sdk.output.output import rs_output_all, rs_output_one, rs_output_one_example
from ketacli.sdk.util import parse_url_params, template_render
from rich.console import Console
from rich.live import Live
from rich.progress import track
from rich.panel import Panel
from ketacli.sdk.chart.plot import Plot
from ketacli.sdk.chart.base import PlotextMixin
from ketacli.sdk.chart.layout import create_layout
from ketacli.sdk.chart.single_value import SingleValueChart
from ketacli.sdk.chart.table import KTable
from ketacli.sdk.chart.no_data import NoDataChart

from pkg_resources import resource_filename

console = Console()


@command
def login(name="keta", endpoint="http://localhost:9000", token=""):
    """Login to ketadb, cache authentication info to ~/.keta/config.yaml

    :param repository: Repository to push to.
    :param -n, --name: The login account name. Defaults to "keta".
    :param -e, --endpoint: The ketadb endpoint. Defaults to "http://localhost:9000".
    :param -t, --token: Your keta api token, create from ketadb webui. Defaults to "".
    """
    do_login(name=name, endpoint=endpoint, token=token)


@command
@arg("name", type=str,
     completer=lambda prefix, **kwd: [x['name'] for x in list_clusters() if x['name'].startswith(prefix)])
def logout(name=None):
    """Logout from ketadb, clear authentication info

    :param -n, --name: logout from which account
    """
    do_logout(name)


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def list(asset_type, groupId=-1, order="desc", pageNo=1, pageSize=10, prefix="", sort="updateTime", fields="",
         format=None, raw=False, lang=None, extra=None, watch=False, interval=3.0):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -l, --pageSize: Limit the page size.
    :param --pageNo: Limit the page number.
    :param --prefix: Fuzzy query filter.
    :param --sort: The field used to order by
    :param --order: The sort order, desc|asc
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --groupId: The resource group id.
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    :param --lang: Choose the language preference of return value
    :param -e, --extra: extra query filter, example: include_defaults=true,flat_settings=true
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    """
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)

    def generate_table():
        req = list_assets_request(
            asset_type, groupId, order, pageNo, pageSize, prefix, sort, lang, **extra_dict)
        resp = request_get(req["path"], req["query_params"],
                           req["custom_headers"]).json()
        output_fields = []
        if len(fields.strip()) > 0:
            output_fields = fields.strip().split(",")
        table = list_output(asset_type, output_fields=output_fields, resp=resp)
        if not table:
            return None
        return format_table(table, format, not raw)

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()
    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any {asset_type}")
        else:
            console.print(table, overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
def admin(asset_type, format=None, extra=None, watch=False, interval=3.0):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    :param -e, --extra: extra query filter, example: include_defaults=true,flat_settings=true
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    """
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)

    def generate_table():
        req = list_admin_request(asset_type, **extra_dict)
        resp = request_get(req["path"], req["query_params"], req["custom_headers"]).json()
        output_fields = []
        table = list_output(asset_type, output_fields=output_fields, resp=resp)
        return format_table(table, format)

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()
    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any {asset_type}")
        else:
            console.print(table)


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def get(asset_type, asset_id, fields="", format=None, lang=None, extra=None):
    """Get asset detail info from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param asset_id: The unique id of asset. (such as id or name...)
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --lang: Choose the language preference of return value
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    extra_args_map['name'] = asset_id

    req = get_asset_by_id_request(
        asset_type=asset_type, asset_id=asset_id, lang=lang, **extra_args_map)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    if format == "json":
        console.print(json.dumps(resp, indent=2, ensure_ascii=False))
        return

    output_fields = []
    if len(fields.strip()) > 0:
        output_fields = fields.strip().split(",")
    table = get_asset_output(output_fields=output_fields, resp=resp)
    table.align = "l"
    if table is None:
        console.print(f"we cannot find any {asset_type}")
    else:
        console.print(format_table(table, format), overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def describe(asset_type, format=None):
    """Describe the schema of asset type

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    """

    req = list_assets_request(asset_type)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    table = describe_output(asset_type, resp=resp)
    if table is None:
        console.print(f"we cannot find any {asset_type}")
    else:
        console.print(format_table(table, format), overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
def search(spl, start=None, end=None, limit=100, format=None, raw=False, watch=False, interval=3.0):
    """Search spl from ketadb

    :param spl: The spl query
    :param --start: The start time. Time format "2024-01-02 10:10:10"
    :param --end: The start time. Time format "2024-01-02 10:10:10"
    :param -l, --limit: The limit size of query result
    :param -f, --format: The output format, table|text|json|csv|html|latex
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    """
    if start is not None:
        start = datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
    if end is not None:
        end = datetime.strptime(end, '%Y-%m-%d %H:%M:%S')

    def generate_table():
        resp = search_spl(spl=spl, start=start, end=end, limit=limit)
        return format_table(search_result_output(resp), format, not raw)

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()

    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any data")
        else:
            console.print(table, overflow="fold")


@command
def plot(spl, start=None, end=None, limit=100, interval=3.0,
         type="line", title=None, x_label="Time", y_label="Value", x_field="_time",
         y_field="value", group_field="", extra=None):
    """plot chart of a single plot.

    :param spl: The spl query
    :param --start: The start time. Time format "2024-01-02 10:10:10"
    :param --end: The start time. Time format "2024-01-02 10:10:10"
    :param -l, --limit: The limit size of query result
    :param --interval: refresh the resource change
    :param -t, --type: plot type, line|bar|scatter
    :param --title: plot title
    :param --x_label: x label
    :param --y_label: y label
    :param -x, --x_field: x field
    :param -y, --y_field: y field
    :param -g, --group_field: group field
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if start is not None:
        start = datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
    if end is not None:
        end = datetime.strptime(end, '%Y-%m-%d %H:%M:%S')
    if not title:
        title = spl
    extra_args_map = {}
    if extra:
        extra_args_map = parse_url_params(extra)

    def generate_table():
        resp = search_spl(spl=spl, start=start, end=end, limit=limit)
        return search_result_output(resp)

    layout = create_layout(rows=1, cols=1, layout_configs=None)

    phase = 0
    with Live(layout, refresh_per_second=0.01) as live:
        while True:
            try:
                data = generate_table()
                plot = Plot(field_x=x_field, field_y=[y_field], data=data, title=title, x_label=x_label,
                            y_label=y_label,
                            field_group=group_field, plot_type=type, **extra_args_map)
                phase += 1
                panel = Panel(PlotextMixin(make_plot=plot.build))
                layout['row0-col0'].update(panel)
                time.sleep(interval)
                live.refresh()
            except KeyboardInterrupt:
                live.stop()
                sys.exit()


@command
@arg("chart", type=str, completer=lambda prefix, **kwd: [
    x.replace('.yaml', '').replace('.yml', '')
    for x in os.listdir(resource_filename('ketacli', "charts")) if x.startswith(prefix)
])
@arg("theme", type=str,
     completer=lambda prefix, **kwd: [
         x for x in ["default", "dark", "clear", "pro", "matrix", "windows",
                     "retro", "elegant", "mature", "dreamland", "grandpa",
                     "salad", "girly", "serious", "sahara", 'scream'] if x.startswith(prefix)
     ])
def dashboard(chart=None, file_path=None, interval=30, theme=""):
    """plot dashboard from yaml file

    :param -c, --chart: The chart name, such as monitor
    :param -f, --file_path: The file path
    :param --interval: refresh the chart change, default 30s
    :param --theme: setting the theme, such as default|dark|clear|pro|matrix|windows|retro|elegant|mature|dreamland|grandpa|salad|girly|serious|sahara|scream
    """
    if file_path is None and chart is None:
        console.print(f"Please specify file path with --file or --chart")
        return
    if chart is not None:
        file_path = os.path.join(resource_filename('ketacli', "charts"), f"{chart}.yaml")
    data = yaml.safe_load(open(file_path))
    layout_config = data.get("layout", {})
    chart_configs = data.get("charts", [])
    layout = create_layout(rows=layout_config.get("rows", 1), cols=layout_config.get("columns", 1),
                           layout_configs=layout_config.get('configs', {}))

    def get_chart(theme=None, live=None):
        for chart_config in chart_configs:
            if not theme:
                theme = chart_config.get("theme", layout_config.get("theme", "dark"))
            chart_type = chart_config.get("type", "line")
            spl = chart_config.get("spl", "")
            position = chart_config.get("position")

            start = chart_config.get("start", None)
            end = chart_config.get("end", None)
            limit = chart_config.get("limit", 500)
            search_data = search_result_output(search_spl(spl=spl, start=start, end=end, limit=limit))

            if not search_data.header:
                title = chart_config.get("title", "")
                panel = NoDataChart(title=title)
                layout[position].update(panel)
            elif chart_type in ["line", "bar", "scatter"]:
                title = chart_config.get("title", "")
                x_field = chart_config.get("x_field", "_time")
                y_field = chart_config.get("y_field", "value")
                y_axis_label = chart_config.get("y_label", "Value")
                x_axis_label = chart_config.get("x_label", "Time")
                group_field = chart_config.get("group_field", "")
                extra_data = chart_config.get("extra_configs", {})
                plot = Plot(field_x=x_field, field_y=y_field, data=search_data, title=title,
                            x_label=x_axis_label, y_label=y_axis_label,
                            field_group=group_field, plot_type=chart_type, theme=theme, **extra_data)
                panel = Panel(PlotextMixin(make_plot=plot.build))
                layout[position].update(panel)
            elif chart_type == "single":
                chart_config_cp = deepcopy(chart_config)
                title = chart_config_cp.pop("title", "")
                single_field = chart_config_cp.pop("single_field", "")
                extra_fields = chart_config_cp.pop("extra_fields", [])
                single_chart = SingleValueChart(search_data, single_field=single_field, title=title,
                                                extra_fields=extra_fields, **chart_config_cp)
                layout[position].update(single_chart)
            elif chart_type == "table":
                title = chart_config.get("title", "")
                kwargs = chart_config.get('columns', {})
                table = KTable(search_data, title=title, **kwargs)
                layout[position].update(table)
            if live:
                live.refresh()
        return layout

    with Live(layout, refresh_per_second=1) as live:
        while True:
            try:
                get_chart(theme, live)
                time.sleep(interval)
            except KeyboardInterrupt:
                live.stop()
                sys.exit()


@command
def insert(repo="default", data=None, file=None):
    """Upload data to specified repo

    :param --repo: The target repo
    :param --data: The json string data [{"raw":"this is text", "host": "host-1"}]
    :param --file: Upload json text from file path.
    """
    if repo is None:
        console.print(f"Please specify target repo with --repo")
        return
    if data is None and file is None:
        console.print(f"Please use --data or --file to specify data to upload")
        return

    if file is not None:
        f = open(file)
        data = f.read()

    query_params = {
        "repo": repo,
    }
    resp = request_post("data", json.loads(data), query_params).json()
    console.print(resp, overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def create(asset_type, name=None, data=None, file=None, extra=None):
    """Create asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name
    :param --data: The json string data {...}
    :param --file: Upload json text from file path.
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if data is None and file is None:
        content = {}
    else:
        content = data
        if file is not None:
            f = open(file)
            content = f.read()
        try:
            content = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')
    req = create_asset_request(asset_type, name, content, **extra_args_map)
    resp = request(req["method"], req["path"], data=req['data']).json()
    console.print(resp, overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def update(asset_type, name=None, operation="update", data=None, file=None, extra=None):
    """Create asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name
    :param -d, --data: The json string data {...}
    :param -f, --file: Upload json text from file path.
    :param -o, --operation: operation type, such as open, close, update, delete
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if data is None and file is None:
        data = {}
    else:
        content = data
        if file is not None:
            f = open(file)
            content = f.read()

        try:
            data = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')
    req = update_asset_request(asset_type, operation, name, data, **extra_args_map)
    resp = request(req["method"], req["path"], data=req['data']).json()
    console.print(resp, overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def delete(asset_type, name=None, data=None, file=None, extra=None):
    """Delete asset

    :param --asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name or id
    :param -d, --data: The json string data {...}
    :param -f, --file: Upload json text from file path.
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if data is None and file is None:
        data = {}
    else:
        content = data
        if file is not None:
            f = open(file)
            content = f.read()

        try:
            data = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return

    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')

    names = name.split(" ")
    for name in names:
        req = delete_asset_request(asset_type, name, data, **extra_args_map)
        resp = request(req["method"], req["path"], data=req['data']).json()
        console.print(resp, overflow="fold")


@command
@arg('type', type=str, completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def rs(type=None, format="table"):
    """Show resource info

    :param -t, --type: The target asset type, such as repo, sourcetype ...
    :param -f, --format: The output format, text, json ...
    """
    resources = get_resources()
    if type is None:
        table = rs_output_all(resources)
    else:
        table = rs_output_one(type, resources.get(type))
    console.print(format_table(table, format=format), overflow="fold")


@command
@arg('name', '-n', '--name', type=str,
     completer=lambda prefix, **kwd: [x['name'] for x in list_clusters() if x['name'].startswith(prefix)])
@arg('operation', type=str,
     completer=lambda prefix, **kwd: [x for x in ['list-clusters', 'set-default', 'delete-cluster'] if
                                      x.startswith(prefix)])
def config(operation, name=None):
    """Show keta cluster info

    :param operation: The target operation, such as list-clusters, set-default, delete-cluster
    :param -n, --name: setting or delete default cluster
    """
    resp = []
    if operation not in ["list-clusters", "set-default", "delete-cluster"]:
        console.print(f"Please specify operation, such as list-clusters, set-default, delete-cluster", style="red")
        return
    if operation == "list-clusters":
        resp = list_clusters()
    elif operation == "set-default":
        resp = set_default_cluster(name)
    elif operation == "delete-cluster":
        resp = delete_cluster(name)
    if not resp:
        console.print("No response")
        exit()
    table = list_output('cluster', [], resp=resp)
    console.print(format_table(table, "table"), overflow="fold")


@command
def mock_data(repo="default", data=None, file=None, number=1):
    """Mock data to specified repo
    :param --repo: The target repo, default: "default"
    :param --data: The json string data default: {"raw":"{{ faker.sentence() }}", "host": "{{ faker.ipv4_private() }}"}
    :param --file: Upload json text from file path.
    :param --number,-n: Number of data, default 1
    """
    if repo is None:
        console.print(f"Please specify target repo with --repo")
        return
    if data is None and file is None:
        console.print(f"Please use --data or --file to specify data to upload")
        return
    if file is not None:
        f = open(file)
        data = f.read()
    query_params = {
        "repo": repo,
    }

    datas = []
    resps = []
    for i in track(range(number), description="Mocking..."):
        text = template_render(data)
        datas.append(json.loads(text))
        if len(datas) == 100 or i == number - 1:  # 发送条件：达到100条数据或到达最后一个数据
            chunk = datas[:100] if len(datas) > 100 else datas
            resps.append(request_post("data", chunk, query_params).json())
            datas = datas[100:] if len(datas) > 100 else []
    console.print(resps, overflow="fold")


@command
def version():
    _version = importlib.metadata.version('ketacli')
    console.print(_version, overflow="fold")


def start():
    try:

        argcomplete.autocomplete(main.parser)
        main()
    except Exception:
        console.print_exception()


if __name__ == "__main__":
    start()



import time
from datetime import datetime
from datetime import timedelta

from .client import request_post
from .client import request_get

JOBS = "jobs"
FIVE_MINUES = timedelta(minutes=5)


def create_search_job(query, start: datetime = None, end: datetime = None, limit=100, timeout_ms=1000):
    if start is None:
        start = datetime.now() - FIVE_MINUES
    if end is None:
        end = datetime.now()
    data = {
        "query": query,
        "startTime": int(start.timestamp() * 1000),
        "endTime": int(end.timestamp() * 1000),
        "collectSize": limit,
        "timeout": timeout_ms,

        "app": "search",
        "preview": False,
        "mode": "smart",
    }
    return request_post(JOBS, data=data, custom_headers={"Content-Type": "application/json"}).json()


def get_search_job_status(jobid):
    return request_get(f"{JOBS}/{jobid}", custom_headers={"Content-Type": "application/json"}).json()


def get_search_job_result(jobid):
    return request_get(f"{JOBS}/{jobid}/results", custom_headers={"Content-Type": "application/json"}).json()


def search_spl(spl, start=None, end=None, limit=100, req_timeout=3000):
    resp = create_search_job(query=spl, start=start, end=end, limit=limit, timeout_ms=req_timeout)
    jobid = ""
    if ("meta" in resp) and resp["meta"]["process"] == 1:
        return resp["result"]
    else:
        jobid = resp["id"]

    while True:
        status = get_search_job_status(jobid)
        if ("process" in status) and status["process"] == 1:
            break
        time.sleep(0.2)
    return get_search_job_result(jobid)


if __name__ == "__main__":
    print(search_spl('search2 repo="*"', limit=1, req_timeout=1000))



from rich.console import Console
from ketacli.sdk.output.format import OutputTable
from rich.table import Table
from rich.panel import Panel
from rich.console import Text
from ketacli.sdk.chart import utils


class KTable:
    def __init__(self, data: OutputTable, title=None, **kwargs):
        self.data = data
        self.title = title
        self.kwargs = kwargs

    def __rich_console__(self, console, options):
        width = options.max_width or console.width
        self.height = options.height or console.height
        table = Table(show_header=True, header_style="bold magenta", width=width, expand=True, padding=0,
                      show_lines=True)

        table_data = utils.sort_values_by_header(self.data.header,  self.data.rows, self.kwargs)

        table_texts = []

        for key, value in table_data.items():
            col_data = table_data[key]
            column_attributes = col_data['attributes']
            style = column_attributes.get("style", "")
            threshold = column_attributes.get("threshold", None)
            _format = column_attributes.get("format", None)
            suffix = column_attributes.get("suffix", "")
            prefix = column_attributes.get("prefix", "")
            enum = column_attributes.get("enum", None)
            title = col_data['attributes'].get('title', key)
            if not col_data['attributes'].get('is_show', True):
                continue
            table.add_column(title, style="blink", justify="center")
            row_texts = []
            for data in col_data['data']:
                if threshold:
                    style = utils.threshold(data, **threshold)
                if _format:
                    data = utils.format(data, type=_format)
                if enum:
                    enum_values = utils.enum(data, **enum)
                    data = enum_values.get("title", data)
                    style = enum_values.get("style", style)

                text = ((f"{prefix}{data}{suffix}", style),)
                text = Text.assemble(*text, justify="center")
                row_texts.append(text)
            table_texts.append(row_texts)

        for row_texts in [x for x in zip(*table_texts)]:
            table.add_row(*row_texts)
        title_text = Text(f"{self.title}({len(self.data.rows)})", justify="center", style="bold")
        panel = Panel(table, expand=True, padding=0, title=title_text, height=self.height, )
        yield panel


if __name__ == '__main__':
    console = Console()
    table = OutputTable(["main", "res", "shard", "pc"], [[1, 2, 3, 100]])
    console.print(KTable(table, title="title", **{
        "main": {"title": "主", "style": "bold red", "threshold": {"green": [0, 80], "red": [80, 100]}},
        "res": {"title": "资源", "style": "bold green"},
        "shard": {"title": "分片", "style": "bold blue"},
        "pc": {"title": "使用率", "style": "bold yellow"}}))



import jinja2
from faker import Faker

faker = Faker()
faker_zh = Faker(locale='zh_CN')


def is_key(key, value_map=None):
    """精确匹配是否存在key

    Args:
        key (str): 字符串key
        value_map (dict, optional): 是否存在该key的字典. Defaults to {}.

    Returns:
        _type_: 最终的key，如果不存在则为None
    """
    if value_map is None:
        value_map = {}
    newkey = key.lower()
    if newkey in value_map:
        return newkey
    return None


def is_fuzzy_key(key, value_map=None):
    """
    检查key或者key的复数形式在map中，并返回最终map中的key
    """
    if value_map is None:
        value_map = {}
    newkey = key.lower()
    if newkey in value_map:
        return newkey
    if newkey.endswith('s'):
        newkey = newkey[:-1]
    else:
        newkey = newkey + 's'
    if newkey in value_map:
        return newkey
    return None


def parse_url_params(url_query):
    params = url_query.lstrip('?')  # 移除开头的问号
    pairs = params.split(',')  # 将参数对分割
    result = {}

    for pair in pairs:
        key, value = pair.split('=')  # 分割键和值
        try:
            value = int(value) if '.' not in value else float(value)  # 尝试转换为整数或浮点数
        except ValueError:
            pass  # 如果转换失败，保留原始字符串值
        result[key] = value  # 添加到结果字典
    return result



def template_render(template, **kwargs):
    temp = jinja2.Template(template)
    temp.globals.update({"faker": faker, "faker_zh": faker_zh})
    faker.phone_number()
    return temp.render(**kwargs)



from rich.panel import Panel
from rich.console import Console
from rich.text import Text
from ketacli.sdk.output.format import OutputTable
from ketacli.sdk.chart import utils

class SingleValueChart:
    def __init__(self, data: OutputTable, single_field, title=None, extra_fields: list = None, **kwargs):
        self.data = data
        self.single_field = single_field
        self.title = title
        self.extra_fields = extra_fields
        self.kwargs = kwargs

    def __rich_console__(self, console, options):
        width = options.max_width or console.width
        self.height = options.height or console.height
        prefix = self.kwargs.get("prefix", "")
        suffix = self.kwargs.get("suffix", "")
        style = self.kwargs.get("style", "")
        threshold = self.kwargs.get("threshold", {})
        _format = self.kwargs.get("format", None)

        index = self.data.header.index(self.single_field)
        single_value = utils.format([str(x[index]) for x in self.data.rows][0], type=_format)
        if threshold:
            style = utils.threshold(single_value, **threshold)
        single_value = f"{prefix}{single_value}{suffix}"
        extra_values = []
        if self.extra_fields:
            for extra_field in self.extra_fields:
                extra_values.append(
                    f"{extra_field}: {[str(x[self.data.header.index(extra_field)]) for x in self.data.rows][0]}")
        extra_value = "\n".join(extra_values)
        padding = ((int(self.height) - (len(self.extra_fields) + 3)) // 2, 0)
        texts = ((single_value, style), "\n", extra_value)
        panel = Panel(Text.assemble(*texts, justify="center"),
                      expand=True, padding=padding, title=self.title, width=width, height=self.height)
        yield panel


if __name__ == '__main__':
    console = Console()
    table = OutputTable(["main", "res", "shard", "pc"], [[1, 2, 3, 100]])
    console.print(SingleValueChart(table, "pc", title="title", extra_fields=["main", "res", "shard"]))



from rich import print
from rich.layout import Layout


def create_layout(rows, cols, layout_configs=None):
    if layout_configs is None:
        layout_configs = {}
    root = Layout(name="root")
    row_layouts = []
    for i in range(rows):
        row_layout = Layout(name=f"row{i}", **layout_configs.get(f'row{i}', {}))
        row_layouts.append(row_layout)
        column_layouts = []
        for j in range(cols):
            col_config = layout_configs.get(f'row{i}-col{j}', {})
            if col_config:
                size = col_config.get("size", None)
                ratio = col_config.get("ratio", None)
                if not size and not ratio:
                    continue
            column_layout = Layout(name=f"row{i}-col{j}", **col_config)
            column_layouts.append(column_layout)
        row_layout.split_row(*column_layouts)
    root.split_column(*row_layouts)
    return root


if __name__ == '__main__':
    configs = {
        "row0": {"size": 0, "ratio": 2},
        "row4": {"size": 0, "ratio": 3},
        "row0-col0": {"size": 0, "ratio": 2},
        "row1-col1": {"size": 0, "ratio": 1},
    }

    layout = create_layout(5, 3, configs)
    print(layout)



from datetime import datetime


def threshold(value: (float, int), **kwargs):
    """
    判断值是否在阈值范围内，返回对应的 key
    threshold = threshold(value, green=(0, 80), red=(80, 100))

    Args:
        value:
        **kwargs:

    Returns:

    """
    if not kwargs:
        return ""
    if not isinstance(value, (float, int)):
        return ""
    for style, threshold_value in kwargs.items():
        if threshold_value[0] is None and value < threshold_value[1]:
            return style
        elif threshold_value[1] is None and value > threshold_value[0]:
            return style
        elif threshold_value[0] <= value <= threshold_value[1]:
            return style


def duration(value: float, type: str):
    if type == "duration_ms":
        milliseconds = int(value)
        seconds, milliseconds = divmod(milliseconds, 1000)
    else:
        seconds = int(value)
    minutes, seconds = divmod(seconds, 60)
    hours, minutes = divmod(minutes, 60)
    days, hours = divmod(hours, 24)

    duration_str = ""
    if days > 0:
        duration_str += f"{days} 天 "
    if hours > 0:
        duration_str += f"{hours} 小时 "
    if minutes > 0:
        duration_str += f"{minutes} 分 "
    duration_str += f"{seconds} 秒"
    return duration_str


def bytes_to_human(value):
    """
    将字节转换为人类可读的格式
    :param value:
    :return:
    """
    if value < 1024:
        return f"{value} B"
    elif value < 1024 * 1024:
        return f"{value / 1024:.2f} KB"
    elif value < 1024 * 1024 * 1024:
        return f"{value / 1024 / 1024:.2f} MB"
    else:
        return f"{value / 1024 / 1024 / 1024:.2f} GB"


def format(value, type=None, **kwargs):
    if not value:
        return value
    if type == "int":
        return int(value)
    elif type == "float":
        return round(float(value), 2)
    elif type == "bytes":
        return bytes_to_human(value)
    elif type == "timestamp_ms":
        return datetime.fromtimestamp(value / 1000).strftime(kwargs.get("format", "%Y-%m-%d %H:%M:%S"))
    elif type == "timestamp_s":
        return datetime.fromtimestamp(value).strftime(kwargs.get("format", "%Y-%m-%d %H:%M:%S"))
    elif type == "duration_ms":
        return duration(value, type="duration_ms")
    elif type == "duration_s":
        return duration(value, type="duration_s")
    elif type == "percentage100":
        return round(float(value), 2)
    elif type == "percentage":
        return round(float(value) * 100, 2)
    else:
        return value


def enum(value, **kwargs):
    if not kwargs or not value:
        return value
    for enum_value, map in kwargs.items():
        if enum_value == value:
            return map
    else:
        return {}


def sort_values_by_header(A, B, C):
    # 将 A 列表和 B 列表构建成字典
    table_dict = dict(zip(A, [pair for pair in zip(*B)]))

    # 根据 C 列表的顺序对字典进行排序
    sorted_values = {}
    for header in C:
        if header in table_dict:
            sorted_values[header] = {"data": table_dict[header], "attributes": C[header]}
            del table_dict[header]  # 删除已处理的键值对

    # 将剩余的键值对放到结果的最后面
    for k, v in table_dict.items():
        sorted_values[k] = {"data": v, "attributes": {}}

    return sorted_values


if __name__ == '__main__':
    print(threshold(value=81, green=(0, 80), red=(80, 100)))
    print(format(value=60000, type="duration_ms"))
    print(format(value=1024 * 1024 * 1024, type="bytes"))
    print(f"Int: {format(222, 'int')}")
    print(f"Float: {format(3.14159, 'float')}")
    print(f"Bytes: {format(len(b'test'), 'bytes')}")
    print(f"Timestamp (ms): {format(1640000000000, 'timestamp_ms')}")
    print(f"Timestamp (s): {format(16400000, 'timestamp_s')}")
    print(f"Duration (ms): {format(16400000, 'duration_ms')}")
    print(f"Duration (s): {format(164000, 'duration_s')}")
    print(f"Percentage100: {format(75.01, 'percentage100')}")
    print(f"Percentage: {format(0.75, 'percentage')}")
    print(enum(value="online", online={"title": "在线", "style": "green"}))
